@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3278 2007-11-29 19:07:04Z shaeger $"
-__version__   = "$Revision: 3278 $"
-__date__      = "$Date: 2007-11-29 14:07:04 -0500 (Thu, 29 Nov 2007) $"
+__id__        = "$Id: braillegenerator.py 3491 2008-01-22 17:46:51Z wwalker $"
+__version__   = "$Revision: 3491 $"
+__date__      = "$Date: 2008-01-22 12:46:51 -0500 (Tue, 22 Jan 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -164,21 +164,10 @@ class BrailleGenerator:
 
         if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
             text = ""
-
-            result = self._script.getAcceleratorAndShortcut(obj)
-
-            accelerator = result[0]
-            #shortcut = result[1]
-
+            result = self._script.getKeyBinding(obj)
+            accelerator = result[2]
             if len(accelerator) > 0:
                 text += "(" + accelerator + ")"
-
-            # [[[TODO: WDW - various stuff preserved while we work out the
-            # desired verbosity here.]]]
-            #
-            #if len(shortcut) > 0:
-            #    text += "(" + shortcut + ")"
-
             return text
         else:
             return None
@@ -441,9 +430,21 @@ class BrailleGenerator:
             regions.append(braille.Region(label + " "))
             focusedRegionIndex = 1
 
-        displayedText = self._script.getDisplayedText(obj)
-        if displayedText:
-            regions.append(braille.Region(displayedText))
+        # Check to see if the text is editable. If so, then we want
+        # to show the text attributes (such as selection -- see bug
+        # 496846 for more details).
+        #
+        textObj = None
+        for child in obj:
+            if child and child.getRole() == pyatspi.ROLE_TEXT:
+                textObj = child
+        if textObj and textObj.getState().contains(pyatspi.STATE_EDITABLE):
+            textRegion = braille.Text(textObj)
+            regions.append(textRegion)
+        else:
+            displayedText = self._script.getDisplayedText(obj)
+            if displayedText:
+                regions.append(braille.Region(displayedText))
 
         regions.append(braille.Region(
             " " + rolenames.getBrailleForRoleName(obj)))
@@ -611,7 +612,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForLabel", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        textRegion = braille.Text(obj,
+                                  self._script.getDisplayedLabel(obj),
+                                  " $l")
+        regions.append(textRegion)
+
+        # We do not want the role at the end of text areas.
+
+        return [regions, textRegion]
 
     def _getBrailleRegionsForList(self, obj):
         """Get the braille for a list.
@@ -1179,6 +1189,8 @@ class BrailleGenerator:
                 regions = [regions, cellFocusedRegion]
                 return regions
 
+        label = None
+
         # [[[TODO: WDW - Attempt to infer the cell type.  There's a
         # bunch of stuff we can do here, such as check the EXPANDABLE
         # state, check the NODE_CHILD_OF relation, etc.  Logged as
@@ -1208,17 +1220,20 @@ class BrailleGenerator:
                         n = table.getColumnAtIndex(obj.getIndexInParent())
                         accHeader = table.getColumnHeader(n)
                         regions[0].append(braille.Region(" "))
-                        regions[0].append(braille.Region(accHeader.name))
+                        label = accHeader.name
+                        regions[0].append(braille.Region(label))
 
                     break
 
+        descendant = self._script.getRealActiveDescendant(obj)
         if len(regions) == 0:
-            regions = self._getDefaultBrailleRegions(
-                              self._script.getRealActiveDescendant(obj))
+            regions = self._getDefaultBrailleRegions(descendant)
         else:
-            [cellRegions, focusRegion] = self._getDefaultBrailleRegions(\
-                self._script.getRealActiveDescendant(obj))
-            regions[0].extend(cellRegions)
+            cellText = self._script.getDisplayedText(descendant)
+            if not cellText or (cellText and label != cellText):
+                [cellRegions, focusRegion] = \
+                    self._getDefaultBrailleRegions(descendant)
+                regions[0].extend(cellRegions)
 
         # Check to see if this table cell contains an icon (image).
         # If yes:
@@ -1393,23 +1408,9 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTerminal", obj)
 
-        title = None
-        frame = self._script.getFrame(obj)
-        if frame:
-            title = frame.name
-        if not title:
-            title = self._script.getDisplayedLabel(obj)
-
-        text = title
-        text = self._script.appendString(
-            text, rolenames.getBrailleForRoleName(obj))
-
         regions = []
-        regions.append(braille.Region(text))
-
         textRegion = braille.Text(obj)
         regions.append(textRegion)
-
         return [regions, textRegion]
 
     def _getBrailleRegionsForToggleButton(self, obj):
