@@ -23,9 +23,9 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py 3099 2007-10-18 20:17:26Z shaeger $"
-__version__   = "$Revision: 3099 $"
-__date__      = "$Date: 2007-10-18 16:17:26 -0400 (Thu, 18 Oct 2007) $"
+__id__        = "$Id: input_event.py 3275 2007-11-29 18:24:26Z richb $"
+__version__   = "$Revision: 3275 $"
+__date__      = "$Date: 2007-11-29 13:24:26 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -42,15 +42,15 @@ SPEECH_EVENT       = "speech"
 
 class InputEvent:
 
-    def __init__(self, type):
+    def __init__(self, eventType):
         """Creates a new input event of the given type.
 
         Arguments:
-        - type: the input event type (one of KEYBOARD_EVENT, BRAILLE_EVENT,
+        - eventType: the input event type (one of KEYBOARD_EVENT, BRAILLE_EVENT,
                 MOUSE_BUTTON_EVENT, MOUSE_MOTION_EVENT, SPEECH_EVENT).
         """
 
-        self.type = type
+        self.type = eventType
 
 class KeyboardEvent(InputEvent):
 
@@ -142,13 +142,13 @@ class InputEventHandler:
                             the function will be called no matter what.
         """
 
-        self._function = function
-        self._description = description
+        self.function = function
+        self.description = description
         self._learnModeEnabled = learnModeEnabled
 
     def __eq__(self, other):
         """Compares one input handler to another."""
-        return (self._function == other._function)
+        return (self.function == other.function)
 
     def processInputEvent(self, script, inputEvent):
         """Processes an input event.  If settings.learnModeEnabled is True,
@@ -169,17 +169,17 @@ class InputEventHandler:
         consumed = False
 
         if settings.learnModeEnabled and self._learnModeEnabled:
-            if self._description:
+            if self.description:
                 # These imports are here to eliminate circular imports.
                 #
                 import braille
                 import speech
-                braille.displayMessage(self._description)
-                speech.speak(self._description)
+                braille.displayMessage(self.description)
+                speech.speak(self.description)
                 consumed = True
         else:
             try:
-                consumed = self._function(script, inputEvent)
+                consumed = self.function(script, inputEvent)
             except:
                 debug.printException(debug.LEVEL_SEVERE)
 
