@@ -25,7 +25,8 @@ __date__      = "$Date:$"
 __copyright__ = "Copyright (c) 2007 Sun Microsystems Inc. and Joanmarie Diggs"
 __license__   = "LGPL"
 
-import orca.atspi as atspi
+import pyatspi
+
 import orca.braille as braille
 import orca.chnames as chnames
 import orca.debug as debug
@@ -47,6 +48,9 @@ from orca.orca_i18n import _ # for gettext support
 
 class Script(default.Script):
 
+    ROLE_DOCUMENT = "Document"
+    ROLE_LINK = "Link"
+
     def __init__(self, app):
         """Creates a new script for the given application.
 
@@ -61,19 +65,24 @@ class Script(default.Script):
         # is "Document".  "Link" is also capitalized in acroread.  We
         # need to make these known to Orca for speech and braille output.
         #
-        self.ROLE_DOCUMENT = "Document"
         rolenames.rolenames[self.ROLE_DOCUMENT] = \
             rolenames.Rolename(self.ROLE_DOCUMENT,
+                               # Translators: short braille for the
+                               # rolename of a document.
+                               #
                                _("doc"),
+                               # Translators: long braille for the
+                               # rolename of a document.
+                               #
                                _("Document"),
+                               # Translators: spoken words for the
+                               # rolename of a document.
+                               #
                                _("document"))
 
-        self.ROLE_LINK = "Link"
         rolenames.rolenames[self.ROLE_LINK] = \
-            rolenames.Rolename(self.ROLE_LINK,
-                                _("lnk"),
-                                _("Link"),
-                                _("link"))
+            rolenames.rolenames[pyatspi.ROLE_LINK]
+
         # To handle the multiple, identical object:text-caret-moved events
         # and possible focus events that result from a single key press
         #
@@ -105,6 +114,18 @@ class Script(default.Script):
                 Script.sayAll,
                 _("Speaks entire document."))
 
+    def getListeners(self):
+        """Sets up the AT-SPI event listeners for this script.
+        """
+        listeners = default.Script.getListeners(self)
+
+        listeners["object:state-changed:checked"]           = \
+            self.onStateChanged
+        listeners["object:state-changed:focused"]           = \
+            self.onStateChanged
+      
+        return listeners
+
     def getDocument(self, locusOfFocus):
         """ Obtains the Document object that contains the locusOfFocus.
 
@@ -116,7 +137,7 @@ class Script(default.Script):
 
         document = None
         obj = locusOfFocus
-        while obj.role != rolenames.ROLE_UNKNOWN:
+        while obj.getRole() != pyatspi.ROLE_UNKNOWN:
             obj = obj.parent
 
         # This is probably it, but the parent of a text object
@@ -124,14 +145,14 @@ class Script(default.Script):
         # has a parent with a role of 'unknown'.  The parent of
         # the Document object is a drawing area.
         #
-        if obj.parent.role == rolenames.ROLE_DRAWING_AREA:
+        if obj.parent.getRole() == pyatspi.ROLE_DRAWING_AREA:
             document = obj
         else:
-            while obj.role != rolenames.ROLE_TABLE:
+            while obj.getRole() != pyatspi.ROLE_TABLE:
                 obj = obj.parent
             # For now, let's assume no nested tables! :-)
             #
-            while obj.role != rolenames.ROLE_UNKNOWN:
+            while obj.getRole() != pyatspi.ROLE_UNKNOWN:
                 obj = obj.parent
             document = obj
 
@@ -151,7 +172,7 @@ class Script(default.Script):
         nodeList = []
         document = self.getDocument(obj)
         while obj != document:
-            nodeList.append(obj.index)
+            nodeList.append(obj.getIndexInParent())
             obj = obj.parent
 
         return nodeList
@@ -175,7 +196,7 @@ class Script(default.Script):
             index = 0
 
         for i in range(index, obj.childCount):
-            child = obj.child(i)
+            child = obj[i]
             for nextObject in self.getNextTextObject(child, nodeList):
                 yield nextObject
             yield child
@@ -200,14 +221,14 @@ class Script(default.Script):
         # SEEM to be consistent. So let's punt until things get properly
         # labeled.
         #
-        rolesList = [rolenames.ROLE_TEXT, \
-                     rolenames.ROLE_UNKNOWN, \
-                     rolenames.ROLE_UNKNOWN, \
-                     rolenames.ROLE_TABLE]
+        rolesList = [pyatspi.ROLE_TEXT,
+                     pyatspi.ROLE_UNKNOWN,
+                     pyatspi.ROLE_UNKNOWN,
+                     pyatspi.ROLE_TABLE]
         if self.isDesiredFocusedItem(obj, rolesList):
             table = obj.parent.parent.parent
             rows = table.childCount
-            columns = table.child(0).childCount
+            columns = table[0].childCount
 
         return [table, rows, columns]
 
@@ -224,8 +245,8 @@ class Script(default.Script):
         # HACK: Again, these things are not labeled or assigned roles,
         # so we're punting for now.
         #
-        column = cell.parent.index + 1
-        row = cell.parent.parent.index + 1
+        column = cell.parent.getIndexInParent() + 1
+        row = cell.parent.parent.getIndexInParent() + 1
 
         return [row, column]
 
@@ -270,14 +291,20 @@ class Script(default.Script):
                    self.getCellCoordinates(oldFocusIsTable, oldFocus)
             [newRow, newCol] = \
                    self.getCellCoordinates(newFocusIsTable, newFocus)
+            # We can't count on being in the first/last cell
+            # of the new row -- only the first/last cell of
+            # the new row that contains data.
+            #
             if newRow != oldRow:
-                # We can't count on being in the first/last cell
-                # of the new row -- only the first/last cell of
-                # the new row that contains data.
+                # Translators: this represents the row and column we're
+                # on in a table.
                 #
                 line = _("row %d, column %d") % (newRow, newCol)
                 speech.speak(line)
             elif newCol != oldCol:
+                # Translators: this represents the column we're
+                # on in a table.
+                #
                 line = _("column %d") % newCol
                 speech.speak(line)
 
@@ -293,16 +320,16 @@ class Script(default.Script):
         """
 
         inFindToolbar = False
-        rolesList = [rolenames.ROLE_DRAWING_AREA, \
-                     rolenames.ROLE_DRAWING_AREA, \
-                     rolenames.ROLE_DRAWING_AREA, \
-                     rolenames.ROLE_TOOL_BAR, \
-                     rolenames.ROLE_PANEL, \
-                     rolenames.ROLE_PANEL, \
-                     rolenames.ROLE_FRAME]
+        rolesList = [pyatspi.ROLE_DRAWING_AREA,
+                     pyatspi.ROLE_DRAWING_AREA,
+                     pyatspi.ROLE_DRAWING_AREA,
+                     pyatspi.ROLE_TOOL_BAR,
+                     pyatspi.ROLE_PANEL,
+                     pyatspi.ROLE_PANEL,
+                     pyatspi.ROLE_FRAME]
 
         try:
-            while obj.role != rolenames.ROLE_DRAWING_AREA:
+            while obj.getRole() != pyatspi.ROLE_DRAWING_AREA:
                 obj = obj.parent
             if self.isDesiredFocusedItem(obj, rolesList):
                 inFindToolbar = True
@@ -328,27 +355,27 @@ class Script(default.Script):
         # We sometimes get focus events for items that don't --
         # or don't yet) have focus.  Ignore these.
         #
-        if (event.source.role == rolenames.ROLE_CHECK_BOX or \
-            event.source.role == rolenames.ROLE_PUSH_BUTTON or \
-            event.source.role == rolenames.ROLE_RADIO_BUTTON) and \
-           not event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
+        if event.source.getRole() in [pyatspi.ROLE_CHECK_BOX,
+                                      pyatspi.ROLE_PUSH_BUTTON,
+                                      pyatspi.ROLE_RADIO_BUTTON] \
+           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
             return
 
-        if not event.source.state.count(atspi.Accessibility.STATE_SHOWING):
+        if not event.source.getState().contains(pyatspi.STATE_SHOWING):
             return
 
-        if not self.findToolbarActive and \
-           event.source.role == rolenames.ROLE_TEXT:
-            if event.source.parent and \
-               (event.source.parent.role == rolenames.ROLE_DRAWING_AREA or \
-                event.source.parent.role == rolenames.ROLE_UNKNOWN):
+        if not self.findToolbarActive \
+           and event.source.getRole() == pyatspi.ROLE_TEXT:
+            parent = event.source.parent
+            if parent and parent.getRole() in [pyatspi.ROLE_DRAWING_AREA,
+                                               pyatspi.ROLE_UNKNOWN]:
                 # We're going to get at least one (and likely several)
                 # caret-moved events which will cause this to get spoken, 
                 # so skip it for now.
                 #
                 return
 
-        if event.source.role == rolenames.ROLE_DRAWING_AREA:
+        if event.source.getRole() == pyatspi.ROLE_DRAWING_AREA:
             # A drawing area can claim focus when visually what has focus is
             # a text object that is a child of the drawing area.  When this
             # occurs, Orca doesn't see the text.  Therefore, try to figure out
@@ -359,26 +386,26 @@ class Script(default.Script):
             debug.println(self.debugLevel, "acroread: Drawing area bug")
             lastKey = None
             try:
-                lastKey = orca_state.lastInputEvent.event_string
+                lastKey = orca_state.lastNonModifierKeyEvent.event_string
             except:
                 pass
-            LOFIndex = orca_state.locusOfFocus.index
+            locusOfFocusIndex = orca_state.locusOfFocus.getIndexInParent()
             childIndex = None
 
             # [[[TODO: JD - These aren't all of the possibilities.  This is
             # very much a work in progress and of testing.]]]
             #
             if lastKey == "Up":
-                childIndex = LOFIndex - 1
+                childIndex = locusOfFocusIndex - 1
             elif lastKey == "Down":
-                childIndex = LOFIndex + 1
+                childIndex = locusOfFocusIndex + 1
             elif lastKey == "Right" or lastKey == "End":
-                childIndex = LOFIndex
+                childIndex = locusOfFocusIndex
             elif lastKey == "Left" or lastKey == "Home":
-                childIndex = LOFIndex
+                childIndex = locusOfFocusIndex
 
             if (childIndex >= 0):
-                child = event.source.child(childIndex)
+                child = event.source[childIndex]
                 event.source = child
 
         default.Script.onFocus(self, event)
@@ -400,12 +427,12 @@ class Script(default.Script):
         # Eliminate unnecessary chattiness related to the Find toolbar.
         #
         if self.findToolbarActive:
-            if newLocusOfFocus.role == rolenames.ROLE_TEXT:
+            if newLocusOfFocus.getRole() == pyatspi.ROLE_TEXT:
                 newText = self.getTextLineAtCaret(newLocusOfFocus)
                 if newText == self.preFindLine:
                     orca.setLocusOfFocus(event, oldLocusOfFocus, False)
                     return
-            if newLocusOfFocus.role == rolenames.ROLE_DRAWING_AREA:
+            if newLocusOfFocus.getRole() == pyatspi.ROLE_DRAWING_AREA:
                 orca.setLocusOfFocus(event, oldLocusOfFocus, False)
                 return
 
@@ -420,19 +447,20 @@ class Script(default.Script):
 
         # Eliminate unnecessary chattiness in the Search panel.
         #
-        if newLocusOfFocus.role == rolenames.ROLE_PUSH_BUTTON and \
-           oldLocusOfFocus and oldLocusOfFocus.role == self.ROLE_LINK and \
-           newLocusOfFocus.name == oldLocusOfFocus.name:
+        if newLocusOfFocus.getRole() == pyatspi.ROLE_PUSH_BUTTON \
+           and oldLocusOfFocus \
+           and oldLocusOfFocus.getRole() == self.ROLE_LINK \
+           and newLocusOfFocus.name == oldLocusOfFocus.name:
             return
 
         # Eliminate general document chattiness.
         #
-        if newLocusOfFocus.role == self.ROLE_DOCUMENT or \
-           newLocusOfFocus.role == rolenames.ROLE_DRAWING_AREA:
+        if newLocusOfFocus.getRole() in [self.ROLE_DOCUMENT,
+                                         pyatspi.ROLE_DRAWING_AREA]:
             orca.setLocusOfFocus(event, newLocusOfFocus, False)
             return
 
-        elif newLocusOfFocus.role == self.ROLE_LINK:
+        elif newLocusOfFocus.getRole() == self.ROLE_LINK:
             # It seems that this will be the only event we will get.  But
             # the default script's onFocus will result in unnecessary
             # verboseness: reporting the drawing area(s) in which this link
@@ -477,7 +505,7 @@ class Script(default.Script):
         # text that may or may NOT have had focus recently.  Sometimes
         # we luck out and it's not showing.
         #
-        if not event.source.state.count(atspi.Accessibility.STATE_SHOWING):
+        if not event.source.getState().contains(pyatspi.STATE_SHOWING):
             return
 
         # Other times, it's showing, but happens to be the text we just
@@ -507,8 +535,8 @@ class Script(default.Script):
         - event: the Event
         """
 
-        if event.type == "object:state-changed:checked" and \
-           event.source.role == rolenames.ROLE_RADIO_BUTTON:
+        if event.type.startswith("object:state-changed:checked") \
+           and event.source.getRole() == pyatspi.ROLE_RADIO_BUTTON:
             # Radio buttons in the Search panel are not automatically
             # selected when you arrow to them.  You have to press Space
             # to select the current radio button.  Watch for this.
@@ -516,9 +544,9 @@ class Script(default.Script):
             orca.visualAppearanceChanged(event, event.source)
             return
 
-        elif event.type == "object:state-changed:focused" and \
-             event.detail1 == 1:
-            if event.source.role == rolenames.ROLE_PUSH_BUTTON:
+        elif event.type.startswith("object:state-changed:focused") \
+             and event.detail1 == 1:
+            if event.source.getRole() == pyatspi.ROLE_PUSH_BUTTON:
                 # Try to minimize chattiness in the Search panel
                 #
                 utterances = \
@@ -530,7 +558,7 @@ class Script(default.Script):
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
-            elif event.source.role == rolenames.ROLE_TEXT:
+            elif event.source.getRole() == pyatspi.ROLE_TEXT:
                 # There's an excellent chance that the Find toolbar just
                 # gained focus.  Check.
                 #
@@ -552,12 +580,17 @@ class Script(default.Script):
 
         if event.source.name == self.findToolbarName:
             self.findToolbarActive = False
-        elif locusOfFocus.text:
-            self.preFindLine = self.getTextLineAtCaret(locusOfFocus)
+        else:
+            try:
+                locusOfFocus.queryText()
+            except NotImplementedError:
+                pass
+            else:
+                self.preFindLine = self.getTextLineAtCaret(locusOfFocus)
 
         default.Script.onWindowDeactivated(self, event)
 
-    def textLines(self, obj, nodeList=None):
+    def textLinesFromNodeList(self, obj, nodeList=None):
         """A generator that can be used to iterate over each line of a
         text object, starting at the caret offset. Overridden here
         because we are not getting any RELATION_FLOWS_TO from acroread.
@@ -588,7 +621,7 @@ class Script(default.Script):
             # doesn't seem to respond to setCaretOffset, so we cannot
             # update our location when sayAll is interrupted or finished.
             #
-            speech.sayAll(self.textLines(document, nodeList),
+            speech.sayAll(self.textLinesFromNodeList(document, nodeList),
                           self.__sayAllProgressCallback)
         else:
             default.Script.sayAll(self, inputEvent)
@@ -608,23 +641,23 @@ class Script(default.Script):
                interface
         """
 
-        if not (obj.parent.role == rolenames.ROLE_DRAWING_AREA or \
-                obj.parent.role == rolenames.ROLE_UNKNOWN):
+        if not (obj.parent.getRole() in [pyatspi.ROLE_DRAWING_AREA,
+                                         pyatspi.ROLE_UNKNOWN]):
             default.Script.sayWord(self, obj)
 
         else:
-            text = obj.text
+            text = obj.queryText()
             offset = text.caretOffset
-            lastKey = orca_state.lastInputEvent.event_string
+            lastKey = orca_state.lastNonModifierKeyEvent.event_string
 
             if lastKey == "Right":
                 penultimateWord = orca_state.lastWord
                 [lastWord, startOffset, endOffset] = \
-                    text.getTextAtOffset(offset,
-                                 atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+                    text.getTextAtOffset(offset, 
+                                         pyatspi.TEXT_BOUNDARY_WORD_START)
                 [word, startOffset, endOffset] = \
                     text.getTextAfterOffset(endOffset+1,
-                                 atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+                                 pyatspi.TEXT_BOUNDARY_WORD_START)
                 if len(penultimateWord) > 0:
                     lastCharPW = penultimateWord[len(penultimateWord) - 1]
                     if lastCharPW == "\n":
@@ -639,7 +672,7 @@ class Script(default.Script):
                 lastWord = orca_state.lastWord
                 [word, startOffset, endOffset] = \
                     text.getTextAtOffset(offset,
-                                 atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+                                 pyatspi.TEXT_BOUNDARY_WORD_START)
                 if len(word) > 0:
                     lastChar = word[len(word) - 1]
                     if lastChar == "\n" and lastWord != word:
