@@ -19,17 +19,16 @@
 
 """Custom script for metacity."""
 
-__id__        = "$Id: metacity.py 2821 2007-09-12 14:28:07Z wwalker $"
-__version__   = "$Revision: 2821 $"
-__date__      = "$Date: 2007-09-12 10:28:07 -0400 (Wed, 12 Sep 2007) $"
+__id__        = "$Id: metacity.py 3354 2007-12-08 02:40:44Z eitani $"
+__version__   = "$Revision: 3354 $"
+__date__      = "$Date: 2007-12-07 21:40:44 -0500 (Fri, 07 Dec 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.braille as braille
 import orca.default as default
-import orca.rolenames as rolenames
-import orca.orca as orca
 import orca.speech as speech
+import pyatspi
 
 from orca.orca_i18n import _
 
@@ -50,6 +49,16 @@ class Script(default.Script):
 
         default.Script.__init__(self, app)
 
+    def getListeners(self):
+        """Sets up the AT-SPI event listeners for this script.
+        """
+        listeners = default.Script.getListeners(self)
+
+        listeners["object:state-changed:showing"]           = \
+            self.onStateChanged
+
+        return listeners
+
     def presentStatusBar(self, obj):
         """Presents information about the metacity status bar."""
 
@@ -67,7 +76,7 @@ class Script(default.Script):
         for app in self.getKnownApplications():
             i = 0
             while i < app.childCount:
-                win = app.child(i)
+                win = app.getChildAtIndex(i)
                 if win is None:
                     print "app error " + app.name
                 elif win.name == obj.name:
@@ -106,7 +115,7 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onNameChanged(self, event)
 
     def onStateChanged(self, event):
@@ -124,7 +133,7 @@ class Script(default.Script):
         # suddenly showing.  Then, we want to present it because we
         # typically do not get onTextInserted events at that time.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onStateChanged(self, event)
         elif (event.type.startswith("object:state-changed:showing")) \
             and event.detail1:
@@ -138,7 +147,7 @@ class Script(default.Script):
         - event: the Event
         """
 
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onTextInserted(self, event)
 
         self.presentStatusBar(event.source)
@@ -152,7 +161,7 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onTextDeleted(self, event)
 
     def onCaretMoved(self, event):
@@ -164,5 +173,5 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onCaretMoved(self, event)
